==> libadalang-23.0.0.tar.gz: libadalang-23.0.0/testsuite/python_support/inline_playground.py <==
#!/usr/bin/env python

import os
import pprint
import re
import sys
from typing import Optional

import libadalang as lal


desc = """
Go over Ada files given as argument, processing any lines starting with --% and
containing a python expression starting with $. For those lines, eval the
expression in a context containing the following variables:

- `node` is the ast node immediately preceding the comment.
- `lal` is the libadalang module.
"""


YELLOW = '\033[33m'

assign_expr_re = re.compile(
    "\s*"
    "(?P<identifier>[a-zA-Z_][a-zA-Z0-9_]*)"
    "\s*"
    "="
    "(?P<expr>.*)"
)


def col(msg, color):
    return msg if not sys.stdout.isatty() else color + msg + '\033[0m'


class InlinePlayground(lal.App):

    def main(self):
        self._output_empty = True
        """
        Track whether we have written to the output. Used to emit line breaks
        when appropriate (see the ``prepare_output`` method).
        """

        # Sort unit by filename to have a deterministic processing order
        for _, unit in sorted(self.units.items()):
            self.process_unit(unit)

    def description(self):
        return desc

    def add_arguments(self):
        self.parser.add_argument(
            '--pretty-out', action='store_true',
            default=os.isatty(sys.stdout.fileno()),
            help='Prettify output for CLI use'
        )
        self.parser.add_argument(
            '--charset', default=None,
            help='Charset to use for source decoding'
        )
        super(InlinePlayground, self).add_arguments()

    def prepare_output(self) -> None:
        if self._output_empty:
            self._output_empty = False
        else:
            print("")

    def process_unit(self, unit):

        def previous_not_trivia(token):
            ret = token
            while ret.is_trivia:
                ret = ret.previous
            return ret

        # Last node for which we evaluated an expression, so that we print
        # "Working on node X" only once per node.
        last_node = None

        # Environment for assignment commands
        local_env = {"lal": lal}

        for tok in unit.iter_tokens():
            if tok.kind == 'Comment' and tok.text.startswith('--%'):
                cmd = tok.text[3:]

                target: Optional[str] = None
                expr: str

                # If the command starts with "<identifier> =", then it's an
                # assignment. Otherwise, we just have an expression to
                # evaluate.
                m = assign_expr_re.match(cmd)
                if m is not None:
                    target = m.group("identifier")
                    expr = m.group("expr")
                else:
                    expr = cmd

                expr = expr.strip()

                # Lookup the node on which to evaluate the expression, emit a
                # heading if the node changed.
                current_node = unit.root.lookup(
                    previous_not_trivia(tok).sloc_range.start
                )
                if current_node != last_node:
                    def heading(with_colors):
                        node_repr = current_node.entity_repr
                        if with_colors:
                            node_repr = col(node_repr, YELLOW)
                        return f"Working on node {node_repr}"
                    self.prepare_output()
                    print(heading(with_colors=True))
                    print("=" * len(heading(with_colors=False)))
                last_node = current_node

                self.prepare_output()
                if target:
                    print("Set '{}' to '{}'".format(
                        col(target, YELLOW), col(expr, YELLOW)
                    ))
                else:
                    print("Eval '{}'".format(col(expr, YELLOW)))

                # Evaluate the expression and print the result. First, update
                # the environment to give access to the current node to the
                # expression to evaluate.
                local_env["node"] = current_node
                value = None
                try:
                    value = eval(expr, None, local_env)
                except lal.PropertyError as pe:
                    print('Exception: {}'.format(
                        ' '.join(str(a) for a in pe.args)
                    ))
                else:
                    # Post-process the result of pprint.pformat so that
                    # non-ASCII or non-printable characters are escaped.
                    value_prefix = 'Result: '
                    indent = '\n' + len(value_prefix) * ' '
                    value_repr = "".join(
                        c if (32 <= ord(c) <= 127
                              or c == '\n') else ascii(c)[1:-1]
                        for c in (pprint.pformat(value)
                                  .replace('\n', indent))
                    )
                    print(value_prefix + col(value_repr, YELLOW))

                # If this was an assigment, do it now for the next expressions
                if target:
                    local_env[target] = value


if __name__ == '__main__':
    InlinePlayground.run()

==> libadalang-2021_20210518.orig.tar.gz: libadalang-2021-20210518/testsuite/python_support/inline_playground.py <==
#!/usr/bin/env python

import os
import sys

import libadalang as lal
from libadalang import _py2to3


desc = """
Go over Ada files given as argument, processing any lines starting with --% and
containing a python expression starting with $. For those lines, eval the
expression in a context containing the following variables:

- `node` is the ast node immediately preceding the comment.
- `lal` is the libadalang module.
"""


YELLOW = '\033[33m'


def col(msg, color):
    return msg if not sys.stdout.isatty() else color + msg + '\033[0m'


class InlinePlayground(lal.App):

    def main(self):
        # Sort unit by filename to have a deterministic processing order
        for _, unit in sorted(self.units.items()):
            self.process_unit(unit)

    def description(self):
        return desc

    def add_arguments(self):
        self.parser.add_argument(
            '--pretty-out', action='store_true',
            default=os.isatty(sys.stdout.fileno()),
            help='Prettify output for CLI use'
        )
        super(InlinePlayground, self).add_arguments()

    def process_unit(self, unit):

        def previous_not_trivia(token):
            ret = token
            while ret.is_trivia:
                ret = ret.previous
            return ret

        for tok in unit.iter_tokens():
            if tok.kind == 'Comment' and tok.text.startswith('--%'):
                expr_text = tok.text[3:].strip()
                if expr_text.startswith('$'):
                    expr_text = expr_text[1:].strip()
                current_node = unit.root.lookup(
                    previous_not_trivia(tok).sloc_range.start
                )
                print("Eval '{}' on node {}".format(
                    col(expr_text, YELLOW),
                    col(current_node.entity_repr, YELLOW)
                ))
                try:
                    value = eval(
                        expr_text, None,
                        {'lal': lal, 'node': current_node}
                    )
                except lal.PropertyError as pe:
                    print('Exception: {}'.format(
                        ' '.join(str(a) for a in pe.args)
                    ))
                else:

                    # Hide discrepancies between Python2 and Python3
                    def canon_repr(value):
                        if isinstance(value, _py2to3.text_type):
                            return _py2to3.text_repr(value)
                        elif isinstance(value, list):
                            return '[{}]'.format(', '.join(
                                canon_repr(v) for v in value
                            ))
                        else:
                            return repr(value)

                    print('Result: {}'.format(col(canon_repr(value), YELLOW)))
                print('')

        print('')


if __name__ == '__main__':
    InlinePlayground.run()
